
// ________________________________________________________________________
// 	file: master slave decoder
//	author: Ajay Dubey 	
// ________________________________________________________________________
// altera message_level Level1 
// altera message_off 10034 10035 10036 10037 10230 10240 10030 
 // _________________________________________________________

 module master_slave_decoder 
      #(parameter NUMSLAVES = 2)
       (input  wire  clk,
	input  wire  reset_n,
	input  wire  avalon_master_write,
	input  wire  avalon_master_read,
	input  wire[31:0] avalon_master_address,
	input  wire[31:0] avalon_master_writedata,
	output wire   avalon_master_waitrequest,
	output wire   avalon_master_readdata_valid,
	output wire[31:0]  avalon_master_readdata,

	output wire  msbridge_if_write,
	output wire  msbridge_if_read,
	output wire[23:0]  msbridge_if_address,
	output wire[31:0]  msbridge_if_writedata,
	output wire[NUMSLAVES-1:0]  msbridge_if_blksel,
	input  wire[NUMSLAVES-1:0]  msbridge_if_waitrequest,
	input  wire[NUMSLAVES-1:0]  msbridge_if_readdata_valid,
	input  wire[32*NUMSLAVES-1:0] msbridge_if_readdata
	);

// _____________________________________________________________________________________
   wire[7:0] blk_address = avalon_master_address[31:24]; // upper 8-bits of byte address
   assign msbridge_if_blksel = vector_with_bitset(blk_address);
   assign msbridge_if_write = avalon_master_write;
   assign msbridge_if_read  = avalon_master_read;
   assign msbridge_if_writedata = avalon_master_writedata;
   assign msbridge_if_address = avalon_master_address[23:0];

 // ________________________________________________
 // 
  reg[31:0] readdata;
  reg readdata_valid, waitrequest; 

  assign avalon_master_readdata = readdata; 
  assign avalon_master_readdata_valid = readdata_valid; 
  assign avalon_master_waitrequest = waitrequest;

   always@(*)
      begin
        case(blk_address)
         8'd0: begin readdata=msbridge_if_readdata[31:0];readdata_valid=msbridge_if_readdata_valid[0];waitrequest=msbridge_if_waitrequest[0]; end
         8'd1: begin readdata=msbridge_if_readdata[63:32];readdata_valid=msbridge_if_readdata_valid[1];waitrequest=msbridge_if_waitrequest[1];end
         `ifdef TWOPORTS
		 8'd2: begin readdata=msbridge_if_readdata[95:64];readdata_valid=msbridge_if_readdata_valid[2];waitrequest=msbridge_if_waitrequest[2];end
		 `endif
         default: begin readdata = 32'd0; readdata_valid = 1'b0; waitrequest = 1'b0; end
        endcase
      end

 	function [NUMSLAVES-1:0] vector_with_bitset;	// output vector with a bit set
 	  input[7:0] index; 				// bit selector
 	          vector_with_bitset = (1'b1 << index); // 2**index 
 	endfunction 



endmodule

