
// ________________________________________________________________________
// 	file: ststem_controller
//	author: Ajay Dubey 	
// ________________________________________________________________________
// altera message_level Level1 
// altera message_off 10034 10035 10036 10037 10230 10240 10030 

module sys_control #(parameter NUMSLAVES = 2) (

	input	wire  reset_n,						// asynchronous reset 
	input	wire  master_if_clk,					// master clock       
	input	wire [NUMSLAVES-1:0] slave_if_clk,			// slave clock 	      (s) 
	output	wire [NUMSLAVES-1:0] slave_if_write,			// slave write 	      (s)
	output	wire [NUMSLAVES-1:0] slave_if_read,			// slave read 	      (s)
	output	wire [24*NUMSLAVES-1:0] slave_if_address,		// slave address      (s)
	output	wire [32*NUMSLAVES-1:0] slave_if_writedata,		// slave writedata    (s)
	input	wire [32*NUMSLAVES-1:0] slave_if_readdata,		// slave read 	      (s)
	input	wire [NUMSLAVES-1:0] slave_if_readdata_valid,		// slave readvalid    (s)
	input	wire [NUMSLAVES-1:0] slave_if_waitrequest		// slave readvalid    (s)
	);
   	wire jtag_if_resetrequest; 
   	wire [31:0] jtag_if_address;
   	wire [3:0] jtag_if_byteenable;
   	wire jtag_if_read; 
   	wire [31:0] jtag_if_readdata; 
   	wire jtag_if_readdatavalid;
   	wire jtag_if_waitrequest;
   	wire jtag_if_write;
   	wire [31:0] jtag_if_writedata;   

	wire jtag_if_readdata_valid;

// Jier : For QII 11.0 (Old) 	
// // _______________________________________________________________________________________________________________
//   	altera_jtag_avalon_master #( .USE_PLI  (0), .PLI_PORT (50000)) JTAG_MASTER (
// // _______________________________________________________________________________________________________________ 
//	.clk  											(master_if_clk),
//	.reset_n     										(reset_n),
//	.resetrequest_from_the_altera_jtag_avalon_master_jtag_interface  			(jtag_if_resetrequest), 
//	.address_from_the_altera_jtag_avalon_master_packets_to_transactions_converter  		(jtag_if_address),
//	.byteenable_from_the_altera_jtag_avalon_master_packets_to_transactions_converter	(jtag_if_byteenable),
//	.read_from_the_altera_jtag_avalon_master_packets_to_transactions_converter 		(jtag_if_read), 
//	.readdata_to_the_altera_jtag_avalon_master_packets_to_transactions_converter		(jtag_if_readdata), 
//	.readdatavalid_to_the_altera_jtag_avalon_master_packets_to_transactions_converter	(jtag_if_readdata_valid),
//	.waitrequest_to_the_altera_jtag_avalon_master_packets_to_transactions_converter 	(jtag_if_waitrequest),
//	.write_from_the_altera_jtag_avalon_master_packets_to_transactions_converter 		(jtag_if_write),
//	.writedata_from_the_altera_jtag_avalon_master_packets_to_transactions_converter 	(jtag_if_writedata)    
//	);

// Jier : For QII 11.1 ownward 
	altera_jtag_avalon_master JTAG_MASTER (
		.master_0_master_reset_reset	(jtag_if_resetrequest),   	// master_0_master_reset.reset
		.master_0_clk_reset_reset		(!reset_n),     				// master_0_clk_reset.reset
		.master_0_master_address		(jtag_if_address),    		// master_0_master.address
		.master_0_master_readdata		(jtag_if_readdata),     	// readdata
		.master_0_master_read			(jtag_if_read),     		// read
		.master_0_master_write			(jtag_if_write),     		// write
		.master_0_master_writedata		(jtag_if_writedata),     	// writedata
		.master_0_master_waitrequest	(jtag_if_waitrequest),   	// waitrequest
		.master_0_master_readdatavalid	(jtag_if_readdata_valid), 	// readdatavalid
		.master_0_master_byteenable		(jtag_if_byteenable),    	// byteenable
		.master_0_clk_clk				(master_if_clk)      		// master_0_clk.clk
	);

     // signals connected to all slaves ports
	wire master_if_write;
	wire master_if_read;
	wire[31:0] master_if_address;
	wire[31:0] master_if_writedata;

     // per slave port signals
   	wire[NUMSLAVES-1:0] master_if_blksel;
  	wire[NUMSLAVES-1:0] master_if_waitrequest;
  	wire[NUMSLAVES-1:0] master_if_readdata_valid; 
  	wire[NUMSLAVES*32-1:0] master_if_readdata; 
 // _________________________________________________________
 //    master front end (for handling busy) and decoder
       master_slave_decoder #(.NUMSLAVES(NUMSLAVES))
       MSDECODER (
 // _________________________________________________________
      .reset_n      	  		(reset_n),
      .clk         			(master_if_clk),
      .avalon_master_write		(jtag_if_write),
      .avalon_master_read		(jtag_if_read),
      .avalon_master_writedata		(jtag_if_writedata),
      .avalon_master_address     	(jtag_if_address[31:0]), 		// byte address  in
      .avalon_master_waitrequest	(jtag_if_waitrequest),
      .avalon_master_readdata		(jtag_if_readdata),
      .avalon_master_readdata_valid	(jtag_if_readdata_valid),

      .msbridge_if_write		(master_if_write),
      .msbridge_if_read			(master_if_read),
      .msbridge_if_address     		(master_if_address[23:0]), 		// byte address lower bits
      .msbridge_if_writedata     	(master_if_writedata[31:0]), 		// write data
      .msbridge_if_blksel   		(master_if_blksel[NUMSLAVES-1:0]),   	// slave select lines
      .msbridge_if_waitrequest		(master_if_waitrequest),		// selected waitrequest for master
      .msbridge_if_readdata_valid	(master_if_readdata_valid),		// selected readdata valid for master
      .msbridge_if_readdata		(master_if_readdata[32*NUMSLAVES-1:0])
    );

 genvar portid;
 generate
      for (portid =0; portid < NUMSLAVES; portid = portid+1)
	 begin: MSBRIDGES
 		// ______________________________________________________________
 		// master slave bridge via asynchronous interface
  	     	   master_slave_bridge  #(.BASE(portid)) 
		   MSBRIDGE (

		  .reset_n      	  	(reset_n),
		  .master_if_clk         	(master_if_clk),
		  .master_if_write       	(master_if_write),
		  .master_if_read        	(master_if_read),
		  .master_if_address     	(master_if_address[23:0]), 		
		  .master_if_writedata   	(master_if_writedata),
		  .master_if_blksel       	(master_if_blksel[portid]),
		  .master_if_readdata   	(master_if_readdata[32*(portid+1)-1:32*portid]),
		  .master_if_readdata_valid 	(master_if_readdata_valid[portid]),
		  .master_if_waitrequest   	(master_if_waitrequest[portid]),
		  
		  .slave_if_clk         	(slave_if_clk[portid]),
		  .slave_if_write       	(slave_if_write[portid]),
		  .slave_if_read        	(slave_if_read[portid]),
		  .slave_if_address     	(slave_if_address[24*(portid+1)-1:24*portid]), 		
		  .slave_if_writedata     	(slave_if_writedata[32*(portid+1)-1:32*portid]), 		
		  .slave_if_readdata     	(slave_if_readdata[32*(portid+1)-1:32*portid]), 		
		  //.slave_if_readdata_valid  	(slave_if_readdata_valid[portid]),
		  .slave_if_waitrequest   	(slave_if_waitrequest[portid])
     		);
	 end
 endgenerate

endmodule

// ____________________________________________________________________________________________
//	reset synchronizer 
// ____________________________________________________________________________________________

// turn off superfluous verilog processor warnings 
// altera message_level Level1 
// altera message_off 10034 10035 10036 10037 10230 10240 10030 

module reset_sync ( clk, data_in, reset_n, data_out) ;

  output data_out;
  input  clk;
  input  data_in;
  input  reset_n;

  reg   data_in_d1 /* synthesis ALTERA_ATTRIBUTE = "{-from \"*\"} CUT=ON ; PRESERVE_REGISTER=ON ; SUPPRESS_DA_RULE_INTERNAL=R101"  */;
  reg   data_out /* synthesis ALTERA_ATTRIBUTE = "PRESERVE_REGISTER=ON ; SUPPRESS_DA_RULE_INTERNAL=R101"  */;
  always @(posedge clk or negedge reset_n)
    begin
      if (reset_n == 0) data_in_d1 <= 0;
      else data_in_d1 <= data_in;
    end


  always @(posedge clk or negedge reset_n)
    begin
      if (reset_n == 0) data_out <= 0;
      else data_out <= data_in_d1;
    end



endmodule

