source common.tcl
source csr_pkg.tcl

open_jtag

puts "=============================================================================="
puts "                      Accessing Ethernet 10G MAC CSR			    "
puts "==============================================================================\n\n"

#==============================================================================
#                       Configuring RX fifo to                
#============================================================================
set_fifo_drop_on_error 1

#============================================================================
#                       Disable Promiscuous mode         

#============================================================================
puts "Frame Decoder Status : [rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0]\n"
set EN_ALLUCAST [rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0]
wr32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0 0x2
puts "Frame Decoder Status : [rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0]\n"

#============================================================================
#                       Configure MAC address       

#============================================================================

#============================================================================
#		By default the mac address is configure as 48'h22446688AACC
#		if you want to configure different MAC address, please change the
#		value 0x00002244 and 0x6688AACC to the value you want. 
#		
#============================================================================

read_address_inserter 
set_address_inserter 1
read_address_inserter

puts "Write to MAC address on transmit path\n"
puts "MAC ADDRESS before write:[rd32 $MAC_BASE_ADDR $ADDRESS_INSERTER_BASE_ADDR 0x04]\n"
puts "MAC ADDRESS before write:[rd32 $MAC_BASE_ADDR $ADDRESS_INSERTER_BASE_ADDR 0x08]\n"
wr32 $MAC_BASE_ADDR $ADDRESS_INSERTER_BASE_ADDR 0x4 0x6688AACC
wr32 $MAC_BASE_ADDR $ADDRESS_INSERTER_BASE_ADDR 0x8 0x00002244
puts "MAC ADDRESS after write:[rd32 $MAC_BASE_ADDR $ADDRESS_INSERTER_BASE_ADDR 0x04]\n"
puts "MAC ADDRESS after write:[rd32 $MAC_BASE_ADDR $ADDRESS_INSERTER_BASE_ADDR 0x08]\n"


puts "Write to MAC address on receive path\n"
puts "MAC ADDRESS before write:[rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x08]\n"
puts "MAC ADDRESS before write:[rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0C]\n"

wr32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x08 0x6688AACC
wr32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0C 0x00002244

puts "MAC ADDRESS after write:[rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x08]\n"
puts "MAC ADDRESS after write:[rd32 $MAC_BASE_ADDR $RX_FRAME_DECODER_BASE_ADDR 0x0C]\n"


# configuration for channel 1
# configure rx fifo
puts "Writing 1 to drop on error register of RX FIFO\n"
wr32 0x30400 0x14 0x0 1


#============================================================================
#                       Disable Promiscuous mode         

#============================================================================
puts "Frame Decoder Status : [rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0]\n"
set EN_ALLUCAST [rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0]
wr32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0 0x2
puts "Frame Decoder Status : [rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0]\n"

#============================================================================
#                       Configure MAC address       

#============================================================================
puts "=============================================================================="
puts "                      Reading Address Inserter CSR			        "
puts "==============================================================================\n\n"
puts "control status : [rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x0]\n"
puts "MAC src addr 0 : [rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x4]\n"
puts "MAC src addr 1 : [rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x8]\n"
    
set offset 0x0

puts "=============================================================================="
puts "                      Write to Control register of Address Inserter  	        "
puts "==============================================================================\n\n"
puts "Write to offset $offset of Address inserter with value of 1\n"
wr32 0x20000 $ADDRESS_INSERTER_BASE_ADDR $offset 1    

puts "=============================================================================="
puts "                      Reading Address Inserter CSR			        "
puts "==============================================================================\n\n"
puts "control status : [rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x0]\n"
puts "MAC src addr 0 : [rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x4]\n"
puts "MAC src addr 1 : [rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x8]\n"

puts "Write to MAC address on transmit path\n"
puts "MAC ADDRESS before write:[rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x04]\n"
puts "MAC ADDRESS before write:[rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x08]\n"
wr32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x4 0x6688AACC
wr32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x8 0x00002244
puts "MAC ADDRESS after write:[rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x04]\n"
puts "MAC ADDRESS after write:[rd32 0x20000 $ADDRESS_INSERTER_BASE_ADDR 0x08]\n"


puts "Write to MAC address on receive path\n"
puts "MAC ADDRESS before write:[rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x08]\n"
puts "MAC ADDRESS before write:[rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0C]\n"

wr32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x08 0x6688AACC
wr32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0C 0x00002244

puts "MAC ADDRESS after write:[rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x08]\n"
puts "MAC ADDRESS after write:[rd32 0x20000 $RX_FRAME_DECODER_BASE_ADDR 0x0C]\n"


# configuration for channel 2
# configure rx fifo
puts "Writing 1 to drop on error register of RX FIFO\n"
wr32 0x50400 0x14 0x0 1


#============================================================================
#                       Disable Promiscuous mode         

#============================================================================
puts "Frame Decoder Status : [rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0]\n"
set EN_ALLUCAST [rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0]
wr32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0 0x2
puts "Frame Decoder Status : [rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0]\n"

#============================================================================
#                       Configure MAC address       

#============================================================================
puts "=============================================================================="
puts "                      Reading Address Inserter CSR			        "
puts "==============================================================================\n\n"
puts "control status : [rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x0]\n"
puts "MAC src addr 0 : [rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x4]\n"
puts "MAC src addr 1 : [rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x8]\n"
    
set offset 0x0

puts "=============================================================================="
puts "                      Write to Control register of Address Inserter  	        "
puts "==============================================================================\n\n"
puts "Write to offset $offset of Address inserter with value of 1\n"
wr32 0x40000 $ADDRESS_INSERTER_BASE_ADDR $offset 1    

puts "=============================================================================="
puts "                      Reading Address Inserter CSR			        "
puts "==============================================================================\n\n"
puts "control status : [rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x0]\n"
puts "MAC src addr 0 : [rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x4]\n"
puts "MAC src addr 1 : [rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x8]\n"

puts "Write to MAC address on transmit path\n"
puts "MAC ADDRESS before write:[rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x04]\n"
puts "MAC ADDRESS before write:[rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x08]\n"
wr32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x4 0x6688AACC
wr32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x8 0x00002244
puts "MAC ADDRESS after write:[rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x04]\n"
puts "MAC ADDRESS after write:[rd32 0x40000 $ADDRESS_INSERTER_BASE_ADDR 0x08]\n"


puts "Write to MAC address on receive path\n"
puts "MAC ADDRESS before write:[rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x08]\n"
puts "MAC ADDRESS before write:[rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0C]\n"

wr32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x08 0x6688AACC
wr32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0C 0x00002244

puts "MAC ADDRESS after write:[rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x08]\n"
puts "MAC ADDRESS after write:[rd32 0x40000 $RX_FRAME_DECODER_BASE_ADDR 0x0C]\n"







close_jtag
