// (C) 2001-2012 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`ifndef TB__SV
`define TB__SV

`include "eth_register_map_params_pkg.sv"
`include "avalon_driver.sv"
`include "avalon_st_eth_packet_monitor.sv"

`timescale 1ps / 1ps

// Top level testbench
module tb;

    // Parameter definition for MAC configuration and packet generation
    parameter UNICAST_ADDR          = 48'h22_44_66_88_AA_CC;
    parameter MULTICAST_ADDR        = 48'h21_44_66_88_AA_CC;
    parameter BROADCAST_ADDR        = 48'hFF_FF_FF_FF_FF_FF;
    parameter PAUSE_MULTICAST_ADDR  = 48'h01_80_C2_00_00_01;
    parameter INVALID_ADDR          = 48'h00_00_00_00_00_00;

    parameter MAC_ADDR              = 48'hEE_CC_88_CC_AA_EE;

    parameter VLAN_INFO             = 16'h0123;
    parameter SVLAN_INFO            = 16'h4567;

    parameter INSERT_PAD            = 1;
    parameter INSERT_CRC            = 0;
    parameter PAUSE_QUANTA          = 16'h0025;

    // Parameter definition for FIFO configuration
    parameter RX_FIFO_DROP_ON_ERROR = 1;

    // Coding for link fault status
    parameter IDLE                  = 2'b00;  

    // modified parameter
    parameter MODE_1G10G_KR_BAR     = 1;   // 1 means 1G/10G line-side mode 0 means 10G base KR mode



    // Get the Avalon interface parameters definition from the package
    import avalon_if_params_pkt::*;

    // Get the register map definition from the package
    import eth_register_map_params_pkg::*;



    // Clock and Reset signals
    wire        xgmii_clk;
    reg         ref_clk_125;
    reg         ref_clk_322;
    reg         reset;
    reg         reset_n;

    wire        avalon_mm_csr_clk;
    wire        avalon_st_rx_clk;
    wire        avalon_st_tx_clk;

    // Clock and reset generation
    initial ref_clk_322 = 1'b0;
    always #1552 ref_clk_322 = ~ref_clk_322;

    initial ref_clk_125 = 1'b0;
    always #4000 ref_clk_125 = ~ref_clk_125;


    initial begin
        reset = 1'b0;
        #10 reset = 1'b1;
        #80000 reset = 1'b0;
    end

    assign reset_n = !reset;


    // Avalon-MM CSR signals
    wire [20:0] avalon_mm_csr_address;
    wire        avalon_mm_csr_read;
    wire [31:0] avalon_mm_csr_readdata;
    wire        avalon_mm_csr_write;
    wire [31:0] avalon_mm_csr_writedata;
    wire        avalon_mm_csr_waitrequest;

    // Avalon-ST RX signals
    wire        avalon_st_rx_startofpacket;
    wire        avalon_st_rx_endofpacket;
    wire        avalon_st_rx_valid;
    wire        avalon_st_rx_ready;
    wire [63:0] avalon_st_rx_data;
    wire [2:0]  avalon_st_rx_empty;
    wire [5:0]  avalon_st_rx_error;

    // Avalon-ST TX signals
    wire        avalon_st_tx_startofpacket;
    wire        avalon_st_tx_endofpacket;
    wire        avalon_st_tx_valid;
    wire        avalon_st_tx_ready;
    wire [63:0] avalon_st_tx_data;
    wire [2:0]  avalon_st_tx_empty;
    wire        avalon_st_tx_error;

    // DUT specific signals
    wire                serial_data_ch0_to_ch1;
    wire                serial_data_ch1_to_ch0;
    wire    [71:0]      xgmii_tx_dc;
    wire    [71:0]      xgmii_rx_dc;
    wire                tx_pcfifo_error_1g_export;
    wire                rx_pcfifo_error_1g_export;

    wire    [15:0]      reconfig_mif_readdata;
    wire                reconfig_mif_read;
    wire    [15:0]      reconfig_mif_address;
    wire                reconfig_mif_waitrequest;

    wire                reconfig_mgmt_waitrequest;
    wire    [31:0]      reconfig_mgmt_writedata;
    wire                reconfig_mgmt_write;
    wire    [6:0]       reconfig_mgmt_address;
    wire    [31:0]      reconfig_mgmt_readdata;
    wire                reconfig_mgmt_read;
    wire                speed_sel;


    // reset controller output
    wire                rst_cntlr_pll_powerdown;
    wire                rst_cntlr_tx_analogreset;
    wire                rst_cntlr_tx_digitalreset;
    wire                rst_cntlr_rx_analogreset;
    wire                rst_cntlr_rx_digitalreset;
    wire                ch1_rst_cntlr_pll_powerdown;
    wire                ch1_rst_cntlr_tx_analogreset;
    wire                ch1_rst_cntlr_tx_digitalreset;
    wire                ch1_rst_cntlr_rx_analogreset;
    wire                ch1_rst_cntlr_rx_digitalreset;
   
    wire                usr_an_lt_reset;
    wire                usr_seq_reset;
    wire [209:0]        reconfig_to_xcvr;
    wire [137:0]        reconfig_from_xcvr;
    wire [209:0]        ch1_reconfig_to_xcvr;
    wire [137:0]        ch1_reconfig_from_xcvr;
    wire                phy_pll_locked;
    wire                ch1_phy_pll_locked;
    wire                pll156M_locked;
    wire                ch1_pll156M_locked;
    wire                hdsk_rc_busy;    
    
    wire                ch1_hdsk_rc_busy;
    wire                ch1_baser_ll_mif_done;
    
    
    wire [5:0]          seq_pcs_mode;
    wire [5:0]          seq_pma_vod;
    wire [4:0]          seq_pma_postap1;
    wire [3:0]          seq_pma_pretap;
    wire [2:0]          tap_to_upd;
    wire                seq_start_rc;
    wire                lt_start_rc;
    wire                en_lcl_rxeq;
    wire                rxeq_done;
    
    wire [5:0]          ch1_seq_pcs_mode;
    wire [5:0]          ch1_seq_pma_vod;
    wire [4:0]          ch1_seq_pma_postap1;
    wire [3:0]          ch1_seq_pma_pretap;
    wire [2:0]          ch1_tap_to_upd;
    wire                ch1_seq_start_rc;
    wire                ch1_lt_start_rc;
    wire                ch1_en_lcl_rxeq;
    wire                ch1_rxeq_done;
   
    wire                lcl_rf;
    wire 	            dmi_mode_en; 
    wire 	            dmi_frame_lock; 
    wire 	            dmi_rmt_rx_ready; 
    wire [5:0]          dmi_lcl_coefl; 
    wire [13:12]        dmi_lcl_coefh; 
    wire 	            dmi_lcl_upd_new; 
    wire 	            dmi_rx_trained; 
    wire 	            dmo_frame_lock; 
    wire 	            dmo_rmt_rx_ready; 
    wire [5:0]          dmo_lcl_coefl; 
    wire [13:12]        dmo_lcl_coefh; 
    wire 	            dmo_lcl_upd_new; 
    wire 	            dmo_rx_trained; 
    
    wire 	            ch1_dmi_mode_en; 
    wire 	            ch1_dmi_frame_lock; 
    wire 	            ch1_dmi_rmt_rx_ready; 
    wire [5:0]          ch1_dmi_lcl_coefl; 
    wire [13:12]        ch1_dmi_lcl_coefh; 
    wire 	            ch1_dmi_lcl_upd_new; 
    wire 	            ch1_dmi_rx_trained; 
    wire 	            ch1_dmo_frame_lock; 
    wire 	            ch1_dmo_rmt_rx_ready; 
    wire [5:0]          ch1_dmo_lcl_coefl; 
    wire [13:12]        ch1_dmo_lcl_coefh; 
    wire 	            ch1_dmo_lcl_upd_new; 
    wire 	            ch1_dmo_rx_trained; 
    
    wire                rx_is_lockedtodata;
    wire                ch1_rx_is_lockedtodata;
    
    wire                tx_cal_busy;
    wire                rx_cal_busy;
    wire                tx_analogreset;
    wire                pll_powerdown_1g;
    wire                pll_powerdown_10g;
    wire                tx_clkout_1g_clk;
    wire                rx_clkout_1g_clk;
    wire                rx_10g_blksync_lock;
    wire                rx_hi_ber;
    wire                generic_pll_rst;
    wire                reconfig_busy_rc;
    wire                rx_coreclkin_1g;
    wire                tx_coreclkin_1g;
    wire                rx_analogreset;
    wire                mode_1g_10gbar;
    wire                ch1_mode_1g_10gbar;
    wire                reset_rc_to_phy;
    wire                rx_syncstatus;
    wire                ch1_rx_syncstatus;
    wire                rx_10g_blk_lock;
    wire                tx_digitalreset;
    wire                rx_digitalreset;
    wire                rx_digitalreset_n;
    wire                tx_digitalreset_n;
    wire                generic_pll_reset;
    wire                baser_ll_mif_done;
    wire                rx_sync_lock;
    wire                ch1_rx_sync_lock;
    
    // st signal for second channel
    wire                lb_avalon_st_startofpacket;
    wire                lb_avalon_st_endofpacket;
    wire                lb_avalon_st_valid;
    wire                lb_avalon_st_ready;
    wire [63:0]         lb_avalon_st_data;
    wire [2:0]          lb_avalon_st_empty;
    wire [5:0]          lb_avalon_st_error;
    
    
    assign pll_locked = phy_pll_locked & pll156M_locked;
    assign pll_powerdown_10g = pll_powerdown_1g;
    
    // status channel 1
    wire 	            ch1_tx_ready;
    wire 	            ch1_rx_ready;    
    wire 	            ch1_pll_locked;    
    wire 	            ch1_tx_cal_busy;    
    wire 	            ch1_rx_cal_busy;    
    wire 	            ch1_lcl_rf; // local device (MAC) Remote Fault = D13
    
    assign ch1_pll_locked = ch1_phy_pll_locked & pll156M_locked;

    wire                ch0_led_link_export;
    wire                ch1_led_link_export;
    wire [1:0]          ch0_link_fault_status_xgmii_rx_data;
    wire [1:0]          ch1_link_fault_status_xgmii_rx_data;
    wire                ch0_rx_data_ready_export;
    wire                ch1_rx_data_ready_export;
    wire                ch0_link_fault_status;
    wire                ch1_link_fault_status;
    wire                datapath_ready;

    // Decode link fault status
    assign ch0_link_fault_status = (ch0_link_fault_status_xgmii_rx_data == IDLE)? 1'b1:1'b0;
    assign ch1_link_fault_status = (ch1_link_fault_status_xgmii_rx_data == IDLE)? 1'b1:1'b0;

    // To indicate MAC is ready
    // Avalon ST driver will only start driving signals when PHY and MAC is ready to sent data.     
    assign datapath_ready = ch0_link_fault_status & ch1_link_fault_status;
    
    
    // DUT instantiation  
        
        altera_eth_10g_mac_base_kr_top dut (
        .mm_clk                                 (ref_clk_125),
        .mm_reset_reset_n                       (reset_n),
    
        .ref_clk_125                            (ref_clk_125),
        .ref_clk_322                            (ref_clk_322),
        .reset_n                                (reset_n),
        .xgmii_clk                              (xgmii_clk),
    
        .ch0_rx_serial_data                     (serial_data_ch1_to_ch0),
        .ch0_tx_serial_data                     (serial_data_ch0_to_ch1),
        .ch1_rx_serial_data                     (serial_data_ch0_to_ch1),
        .ch1_tx_serial_data                     (serial_data_ch1_to_ch0),
    
        .ch0_avalon_st_sink_startofpacket       (avalon_st_tx_startofpacket),
        .ch0_avalon_st_sink_endofpacket         (avalon_st_tx_endofpacket),
        .ch0_avalon_st_sink_valid               (avalon_st_tx_valid),
        .ch0_avalon_st_sink_ready               (avalon_st_tx_ready),
        .ch0_avalon_st_sink_data                (avalon_st_tx_data),
        .ch0_avalon_st_sink_empty               (avalon_st_tx_empty),
        .ch0_avalon_st_sink_error               (avalon_st_tx_error),
    
        .ch0_avalon_st_source_startofpacket     (avalon_st_rx_startofpacket),
        .ch0_avalon_st_source_endofpacket       (avalon_st_rx_endofpacket),
        .ch0_avalon_st_source_valid             (avalon_st_rx_valid),
        .ch0_avalon_st_source_ready             (avalon_st_rx_ready),
        .ch0_avalon_st_source_data              (avalon_st_rx_data),
        .ch0_avalon_st_source_empty             (avalon_st_rx_empty),
        .ch0_avalon_st_source_error             (avalon_st_rx_error),
    
        .ch1_avalon_st_sink_startofpacket       (lb_avalon_st_startofpacket),
        .ch1_avalon_st_sink_endofpacket         (lb_avalon_st_endofpacket),
        .ch1_avalon_st_sink_valid               (lb_avalon_st_valid),
        .ch1_avalon_st_sink_ready               (lb_avalon_st_ready),
        .ch1_avalon_st_sink_data                (lb_avalon_st_data),
        .ch1_avalon_st_sink_empty               (lb_avalon_st_empty),
        .ch1_avalon_st_sink_error               (lb_avalon_st_error),
    
        .ch1_avalon_st_source_startofpacket     (lb_avalon_st_startofpacket),
        .ch1_avalon_st_source_endofpacket       (lb_avalon_st_endofpacket),
        .ch1_avalon_st_source_valid             (lb_avalon_st_valid),
        .ch1_avalon_st_source_ready             (lb_avalon_st_ready),
        .ch1_avalon_st_source_data              (lb_avalon_st_data),
        .ch1_avalon_st_source_empty             (lb_avalon_st_empty),
        .ch1_avalon_st_source_error             (lb_avalon_st_error),
    
        .avalon_mm_csr_address                  (avalon_mm_csr_address[19:0]),
        .avalon_mm_csr_waitrequest              (avalon_mm_csr_waitrequest),
        .avalon_mm_csr_read                     (avalon_mm_csr_read),       
        .avalon_mm_csr_readdata                 (avalon_mm_csr_readdata),   
        .avalon_mm_csr_write                    (avalon_mm_csr_write),      
        .avalon_mm_csr_writedata                (avalon_mm_csr_writedata),

        .ch0_led_link_export                    (ch0_led_link_export),
        .ch1_led_link_export                    (ch1_led_link_export),
        .ch0_link_fault_status_xgmii_rx_data    (ch0_link_fault_status_xgmii_rx_data),
        .ch1_link_fault_status_xgmii_rx_data    (ch1_link_fault_status_xgmii_rx_data),
        .ch0_rx_data_ready_export               (ch0_rx_data_ready_export),
        .ch1_rx_data_ready_export               (ch1_rx_data_ready_export)
        
        );
		defparam dut.MODE_1G10G_KR_BAR = MODE_1G10G_KR_BAR;
        

    // Assign clock signals
    assign avalon_mm_csr_clk    = ref_clk_125;
    assign avalon_st_tx_clk     = xgmii_clk;
    assign avalon_st_rx_clk     = xgmii_clk;

    // Avalon-MM and Avalon-ST signals driver
    avalon_driver U_AVALON_DRIVER (
		.avalon_mm_csr_clk          (avalon_mm_csr_clk),
		.avalon_st_rx_clk           (avalon_st_rx_clk),
		.avalon_st_tx_clk           (avalon_st_tx_clk),

        .reset                      (reset),

        .avalon_mm_csr_address      (avalon_mm_csr_address),
		.avalon_mm_csr_read         (avalon_mm_csr_read),
		.avalon_mm_csr_readdata     (avalon_mm_csr_readdata),
		.avalon_mm_csr_write        (avalon_mm_csr_write),
		.avalon_mm_csr_writedata    (avalon_mm_csr_writedata),
		.avalon_mm_csr_waitrequest  (avalon_mm_csr_waitrequest),

        .avalon_st_rx_startofpacket (avalon_st_rx_startofpacket),
		.avalon_st_rx_endofpacket   (avalon_st_rx_endofpacket),
		.avalon_st_rx_valid         (avalon_st_rx_valid),
		.avalon_st_rx_ready         (avalon_st_rx_ready),
		.avalon_st_rx_data          (avalon_st_rx_data),
		.avalon_st_rx_empty         (avalon_st_rx_empty),
		.avalon_st_rx_error         (avalon_st_rx_error),

        .avalon_st_tx_startofpacket (avalon_st_tx_startofpacket),
		.avalon_st_tx_endofpacket   (avalon_st_tx_endofpacket),
		.avalon_st_tx_valid         (avalon_st_tx_valid),
		.avalon_st_tx_ready         (avalon_st_tx_ready),
		.avalon_st_tx_data          (avalon_st_tx_data),
		.avalon_st_tx_empty         (avalon_st_tx_empty),
		.avalon_st_tx_error         (avalon_st_tx_error)
	);



    // Ethernet packet monitor on Avalon-ST RX path
    avalon_st_eth_packet_monitor #(
		.ST_ERROR_W                 (AVALON_ST_RX_ST_ERROR_W)
    ) U_MON_RX (
		.clk                        (avalon_st_rx_clk),
        .reset                      (reset),

        .startofpacket              (avalon_st_rx_startofpacket),
		.endofpacket                (avalon_st_rx_endofpacket),
		.valid                      (avalon_st_rx_valid),
		.ready                      (avalon_st_rx_ready),
		.data                       (avalon_st_rx_data),
		.empty                      (avalon_st_rx_empty),
		.error                      (avalon_st_rx_error)
	);

    

    // Ethernet packet monitor on Avalon-ST TX path
    avalon_st_eth_packet_monitor #(
		.ST_ERROR_W (AVALON_ST_TX_ST_ERROR_W)
    ) U_MON_TX (
		.clk                        (avalon_st_tx_clk),
        .reset                      (reset),

        .startofpacket              (avalon_st_tx_startofpacket),
		.endofpacket                (avalon_st_tx_endofpacket),
		.valid                      (avalon_st_tx_valid),
		.ready                      (avalon_st_tx_ready),
		.data                       (avalon_st_tx_data),
		.empty                      (avalon_st_tx_empty),
		.error                      (avalon_st_tx_error)
	);



    // Variable to store data read from CSR interface
    bit [31:0] readdata;

    // Control the testbench flow and driving signals by calling Avalon BFM Driver
    initial begin

	@(negedge reset);

    if (MODE_1G10G_KR_BAR == 1 ) begin

        // Run line-side mode test

        // wait for offset cancellation to complete
        while (!(ch0_rx_data_ready_export & ch1_rx_data_ready_export)) @(posedge xgmii_clk);

        // Configure PHY
        //change speed to 10g base-R for ch 0
        U_AVALON_DRIVER.avalon_mm_csr_wr(RECONFIG_SPEED_ADDR, 32'b100);
 
        //change speed to 10g base-R for ch 1
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_RECONFIG_SPEED_ADDR, 32'b100);


        // Configure the MAC for channel 0
        // Enable source address insertion on TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(TX_ADDRESS_INSERT_CONTROL_ADDR, 1);

        // Configure unicast address for TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, MAC_ADDR[31:0]);
        U_AVALON_DRIVER.avalon_mm_csr_wr(TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, MAC_ADDR[47:32]);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(TX_ADDRESS_INSERT_CONTROL_ADDR, readdata);

        U_AVALON_DRIVER.avalon_mm_csr_rd(TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, readdata);
        $display("TX Source Address [47:32]         = 0x%x", readdata[15:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, readdata);
        $display("TX Source Address [31:0]          = 0x%x", readdata[31:0]);

        // Configure the RX FIFO
        U_AVALON_DRIVER.avalon_mm_csr_wr(RX_FIFO_DROP_ON_ERROR_ADDR, RX_FIFO_DROP_ON_ERROR);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(RX_FIFO_DROP_ON_ERROR_ADDR, readdata);
        $display("RX FIFO Drop on Error Enable      = %0d", readdata[0]);
        
        // configure the MAC for channel 1
        // Enable source address insertion on TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_TX_ADDRESS_INSERT_CONTROL_ADDR, 1);

        // Configure unicast address for TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, MAC_ADDR[31:0]);
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, MAC_ADDR[47:32]);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_TX_ADDRESS_INSERT_CONTROL_ADDR, readdata);

        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, readdata);
        $display("channel 1 TX Source Address [47:32]         = 0x%x", readdata[15:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, readdata);
        $display("channel 1 TX Source Address [31:0]          = 0x%x", readdata[31:0]);

        // Configure the RX FIFO
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_RX_FIFO_DROP_ON_ERROR_ADDR, RX_FIFO_DROP_ON_ERROR);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_RX_FIFO_DROP_ON_ERROR_ADDR, readdata);
        $display("channel 1 RX FIFO Drop on Error Enable      = %0d", readdata[0]);


        repeat(10000) @(posedge xgmii_clk);


        // Wait for MAC + PHY to be ready after dynamic reconfiguration
        // read PCS status register, wait 10G base R to be ready to receive data
        U_AVALON_DRIVER.avalon_mm_csr_rd(PHY_BASER_BLOCK_LOCK_ADDR, readdata);
        $display("Rx_DATA_READY ch0         = 0x%x", readdata[31:0]);        
        while (readdata[7] != 1) begin
            U_AVALON_DRIVER.avalon_mm_csr_rd(PHY_BASER_BLOCK_LOCK_ADDR, readdata);
        end
        
        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_PHY_BASER_BLOCK_LOCK_ADDR, readdata);
        $display("Rx_DATA_READY ch1         = 0x%x", readdata[31:0]);        
        while (readdata[7] != 1) begin
            U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_PHY_BASER_BLOCK_LOCK_ADDR, readdata);
        end


        // Wait for MAC to be ready before sending MAC packets
        while (!(datapath_ready)) @(posedge xgmii_clk);

        // Send Ethernet packet through Avalon-ST TX path
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        // U_AVALON_DRIVER.avalon_st_transmit_pause_frame(PAUSE_MULTICAST_ADDR, MAC_ADDR, PAUSE_QUANTA, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_vlan_frame(MULTICAST_ADDR, MAC_ADDR, VLAN_INFO, 1518, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 1518, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_svlan_frame(BROADCAST_ADDR, MAC_ADDR, VLAN_INFO, SVLAN_INFO, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_vlan_frame(UNICAST_ADDR, MAC_ADDR, VLAN_INFO, 500, INSERT_PAD, INSERT_CRC);
        //U_AVALON_DRIVER.avalon_st_transmit_pause_frame(MAC_ADDR, MAC_ADDR, PAUSE_QUANTA, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_svlan_frame(MAC_ADDR, INVALID_ADDR, VLAN_INFO, SVLAN_INFO, 1518, INSERT_PAD, INSERT_CRC);


        repeat(12500) @(posedge xgmii_clk);

        // Display the collected statistics of the MAC channel 0
        $display("\n-------------");
        $display("TX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(TX_STATISTICS_ADDR);

        $display("\n-------------");
        $display("RX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(RX_STATISTICS_ADDR);
        
        repeat(500) @(posedge xgmii_clk);
        // Display the collected statistics of the MAC channel 1
        $display("\n-------------");
        $display("TX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(ch1_TX_STATISTICS_ADDR);

        $display("\n-------------");
        $display("RX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(ch1_RX_STATISTICS_ADDR);

        
        repeat(2500) @(posedge xgmii_clk);

        
        // Configuring PHY to 1G data rate
        // change speed to 1g channel 0
        U_AVALON_DRIVER.avalon_mm_csr_wr(RECONFIG_SPEED_ADDR, 32'b010);

        // change speed to 1g channel 1
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_RECONFIG_SPEED_ADDR, 32'b010);
        
        // disable autonegotiation clause 37
        // disable on channel 0
        U_AVALON_DRIVER.avalon_mm_csr_wr(PHY_GIGE_AN_ADDR, 32'b0);
        $display("control register ch0        = 0x%x", readdata[31:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(PHY_GIGE_AN_ADDR, readdata);
        $display("control register ch0        = 0x%x", readdata[31:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(PHY_GIGE_STATUS_ADDR, readdata);
        $display("auto negotiation status ch0        = 0x%x", readdata[31:0]);
        
        // disable on channel 1
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_PHY_GIGE_AN_ADDR, 32'b0);
        $display("control register ch1        = 0x%x", readdata[31:0]); 

        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_PHY_GIGE_AN_ADDR, readdata);
        $display("control register ch1        = 0x%x", readdata[31:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_PHY_GIGE_STATUS_ADDR, readdata);
        $display("auto negotiation status ch1         = 0x%x", readdata[31:0]);
        
        repeat(8000) @(posedge xgmii_clk);

        // Wait for MAC and PHY to be ready before sending MAC packets
        // read PCS status register
        U_AVALON_DRIVER.avalon_mm_csr_rd(PHY_GIGE_STATUS_ADDR, readdata);
        $display("Link status channel 0         = 0x%x", readdata[31:0]);
        // wait for link status to up
        while (readdata[2] != 1) begin
            U_AVALON_DRIVER.avalon_mm_csr_rd(PHY_GIGE_STATUS_ADDR, readdata);
        end
        
        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_PHY_GIGE_STATUS_ADDR, readdata);
        $display("Link status channel 1         = 0x%x", readdata[31:0]);
        // wait for link status to up
        while (readdata[2] != 1) begin
            U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_PHY_GIGE_STATUS_ADDR, readdata); 
        end 

        while (!(datapath_ready & ch0_led_link_export & ch1_led_link_export)) @(posedge xgmii_clk);


        // Send Ethernet packet through Avalon-ST TX path
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_vlan_frame(MULTICAST_ADDR, MAC_ADDR, VLAN_INFO, 1518, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 1518, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_svlan_frame(BROADCAST_ADDR, MAC_ADDR, VLAN_INFO, SVLAN_INFO, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_vlan_frame(UNICAST_ADDR, MAC_ADDR, VLAN_INFO, 500, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_svlan_frame(MAC_ADDR, INVALID_ADDR, VLAN_INFO, SVLAN_INFO, 1518, INSERT_PAD, INSERT_CRC);



        // Wait until packet loopback on Avalon-ST RX path
        repeat(31200) @(posedge xgmii_clk);



        // Display the collected statistics of the MAC for channel 0
        $display("\n-------------");
        $display("TX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(TX_STATISTICS_ADDR);

        $display("\n-------------");
        $display("RX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(RX_STATISTICS_ADDR);
        
        // Display the collected statistics of the MAC for channel 1
        $display("\n-------------");
        $display("TX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(ch1_TX_STATISTICS_ADDR);

        $display("\n-------------");
        $display("RX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(ch1_RX_STATISTICS_ADDR);

        end
    else
        begin
        
        // Run backplane mode test

        // No reconfiguration needed for PHY

        // Configure the MAC
        // Enable source address insertion on TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(TX_ADDRESS_INSERT_CONTROL_ADDR, 1);

        // Configure unicast address for TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, MAC_ADDR[31:0]);
        U_AVALON_DRIVER.avalon_mm_csr_wr(TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, MAC_ADDR[47:32]);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(TX_ADDRESS_INSERT_CONTROL_ADDR, readdata);
        $display("TX Source Address Insert Enable   = %0d", readdata[0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, readdata);
        $display("TX Source Address [47:32]         = 0x%x", readdata[15:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, readdata);
        $display("TX Source Address [31:0]          = 0x%x", readdata[31:0]);

        // Configure the RX FIFO
        U_AVALON_DRIVER.avalon_mm_csr_wr(RX_FIFO_DROP_ON_ERROR_ADDR, RX_FIFO_DROP_ON_ERROR);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(RX_FIFO_DROP_ON_ERROR_ADDR, readdata);
        $display("RX FIFO Drop on Error Enable      = %0d", readdata[0]);

        // configure the MAC for channel 1
        // Enable source address insertion on TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_TX_ADDRESS_INSERT_CONTROL_ADDR, 1);

        // Configure unicast address for TX
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, MAC_ADDR[31:0]);
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, MAC_ADDR[47:32]);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_TX_ADDRESS_INSERT_CONTROL_ADDR, readdata);
        $display("channel 1 TX Source Address Insert Enable   = %0d", readdata[0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_1_ADDR, readdata);
        $display("channel 1 TX Source Address [47:32]         = 0x%x", readdata[15:0]);

        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_TX_ADDRESS_INSERT_UCAST_MAC_ADD_0_ADDR, readdata);
        $display("channel 1 TX Source Address [31:0]          = 0x%x", readdata[31:0]);

        // Configure the RX FIFO
        U_AVALON_DRIVER.avalon_mm_csr_wr(ch1_RX_FIFO_DROP_ON_ERROR_ADDR, RX_FIFO_DROP_ON_ERROR);

        // Read the configured registers
        U_AVALON_DRIVER.avalon_mm_csr_rd(ch1_RX_FIFO_DROP_ON_ERROR_ADDR, readdata);
        $display("channel 1 RX FIFO Drop on Error Enable      = %0d", readdata[0]);
        
        repeat(50000) @(posedge xgmii_clk);
        
        // Check that auto-negotiation clause 73 is completed
        U_AVALON_DRIVER.avalon_mm_csr_rd(20'h80308, readdata);
        $display("check the auto-negotiation status ch0         = 0x%x", readdata[31:0]);
        while (readdata[2] != 1) begin
            U_AVALON_DRIVER.avalon_mm_csr_rd(20'h80308, readdata); 
        end

       
        // Wait for MAC to be ready before sending MAC packets
        while (!(datapath_ready)) @(posedge xgmii_clk); 


        // Send Ethernet packet through Avalon-ST TX path
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_vlan_frame(MULTICAST_ADDR, MAC_ADDR, VLAN_INFO, 1518, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_data_frame(MAC_ADDR, MAC_ADDR, 1518, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_svlan_frame(BROADCAST_ADDR, MAC_ADDR, VLAN_INFO, SVLAN_INFO, 64, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_vlan_frame(UNICAST_ADDR, MAC_ADDR, VLAN_INFO, 500, INSERT_PAD, INSERT_CRC);
        U_AVALON_DRIVER.avalon_st_transmit_svlan_frame(MAC_ADDR, INVALID_ADDR, VLAN_INFO, SVLAN_INFO, 1518, INSERT_PAD, INSERT_CRC);


        repeat(22500) @(posedge xgmii_clk);
        // Display the collected statistics of the MAC
        $display("\n-------------");
        $display("TX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(TX_STATISTICS_ADDR);

        $display("\n-------------");
        $display("RX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(RX_STATISTICS_ADDR);

        // Display the collected statistics of the MAC for channel 1
        $display("\n-------------");
        $display("TX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(ch1_TX_STATISTICS_ADDR);

        $display("\n-------------");
        $display("RX Statistics");
        $display("-------------");
        U_AVALON_DRIVER.display_eth_statistics(ch1_RX_STATISTICS_ADDR);
        
         
        end


        // Simulation ended
        $display("\n\nSimulation Ended\n");
        $finish();
    end


endmodule

`endif
